
import pandas as pd



def series(df, options, *args, **kwargs):
    idx = df.index
    col = df.columns
    data = df.values
    assert(isinstance(idx, pd.core.index.Index))
    
    series = []
    for k, c in enumerate(col):
        if df[c].dtype.kind in 'fib':
            v = data[:, k]
            sec = c in kwargs.get('secondary_y', [])
            d = {
                'name': c if not sec else c + ' (right)',
                'yAxis': int(sec),
                'data': [[idx[q], v[q]] for q in range(len(v))],             
            }
            if c in kwargs.get('color', []):
                d['color'] = kwargs['color'].get(c)
            if kwargs.get('dashStyle', []):
                d['dashStyle'] = kwargs['dashStyle'].get(c, 'Solid')
            series.append(d)
    return series




def series_drilldown(df, options, *args, **kwargs):
    idx = df.index
    col = df.columns
    assert(isinstance(idx, pd.core.index.MultiIndex))
    assert(len(idx.levshape)==2)
    assert(len(col)==1)
    assert(df[col[0]].dtype.kind in 'if')
    
    levone = list(idx.levels[0])
    data = []
    drilldownSeries = []
    for c in levone:
        dfs = df.xs(c)
        ii = dfs.index.values.flatten()
        vv = dfs.values.flatten()
        
        d1 = {
            'name': c,
            'y': dfs.sum().values[0],
            'drilldown': c if len(dfs)>1 else None,
        }
        data.append(d1)

        if len(dfs)>1:
            d2 = {
                'name': c,
                'data': [[str(ii[q]), vv[q]] for q in range(len(ii))],
                'id': c,                
            }
            drilldownSeries.append(d2)

    series = [{'name': col[0],'data': data, 'colorByPoint': True}]
    return series, drilldownSeries






def series_scatter(df, options, *args, **kwargs):
    idx = df.index
    col = df.columns
    assert(isinstance(idx, pd.core.index.MultiIndex))
    assert(len(idx.levshape)==2)
    assert(len(col)==1)
    assert(df[col[0]].dtype.kind in 'iO')
        
    data = df.values.flatten()
    elmt = list(set(data))
    color = kwargs.get('color', {})
    series = []
    
    for e in elmt:
        dfs = df[df.iloc[:, 0]==e]
        idx = dfs.index
        series.append({'animation': False,
                       'name': e,
                       'color': color.get(e, None),
                       'data': [[idx[k][0], idx[k][1]] for k in range(len(dfs))],
                      })
    return series







def series_bubble(df, options, *args, **kwargs):
    idx = df.index
    col = df.columns
    assert(isinstance(idx, pd.core.index.MultiIndex))
    assert(len(idx.levshape)==3)
    assert(len(col)==1)
    assert(df[col[0]].dtype.kind in 'fib')

    names = list(idx.levels[0])
    color = kwargs.get('color', {})
    series = []
    for s in names:
        dfs = df.xs(s)
        v = dfs.values.flatten()
        idxs = dfs.index
        d = {
            'name': s,
            'data': [[idxs[q][0], idxs[q][1], v[q]] for q in range(len(v))],             
        }
        if s in kwargs.get('color', []):
            d['color'] = kwargs['color'].get(s)

        series.append(d)
        
    return series
    



def series_treemap(df, options, *args, **kwargs):
    idx = df.index
    col = df.columns
    assert(isinstance(idx, pd.core.index.MultiIndex))
    assert(df[col[0]].dtype.kind in 'fi')
        
    data = df.values
    elmt = list(set(data))
    color = kwargs.get('color', {})
    series = []
    
    for e in elmt:
        dfs = df[df.iloc[:, 0]==e]
        idx = dfs.index
        series.append({'animation': False,
                       'name': e,
                       'color': color.get(e, None),
                       'data': [[idx[k][0], idx[k][1]] for k in range(len(dfs))],
                      })
    return series







import os
import re
from jinja2 import Environment

from _config import SCRIPT_DIR
from _img import image_src


def get_path(filename):
    _dir = os.path.dirname(__file__)
    path = os.path.join(_dir, SCRIPT_DIR, filename)
    return path


def remove_comments_js(string):
    # remove all streamed comments (/*COMMENT */) from string
    string = re.sub(re.compile("/\*.*?\*/",re.DOTALL ) ,"" ,string) 
    # remove all singleline comments (//COMMENT\n ) from string
    string = re.sub(re.compile("//.*?\n" ) ,"" ,string) 
    return string   


def load_script(filename, js=True):
    path = get_path(filename)
    with open(path) as f:
        contents = f.read()
        if js:
            contents = remove_comments_js(contents)
        return contents#.replace('\n', '')


def from_template(contents, **kwargs):
    contents = Environment().from_string(contents).render(**kwargs)
    return contents


TOOLTIP_HEADER_FORMAT = '<b>{series.name}</b><br>'
TOOLTIP_POINT_FORMAT_PERCENT = '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b> ({point.change}%)<br/>'
TOOLTIP_POINT_FORMAT_BASIC = '<span style="color:{series.color}">{series.name}</span>: <b>{point.y}</b><br/>'


TOOLTIP_POSITIONER_CENTER_TOP = load_script('tooltip_positioner_center_top.js')
TOOLTIP_POSITIONER_LEFT_TOP = load_script('tooltip_positioner_left_top.js')
TOOLTIP_POSITIONER_RIGHT_TOP = load_script('tooltip_positioner_right_top.js')

FORMATTER_PERCENT = load_script('formatter_percent.js')
FORMATTER_BASIC = load_script('formatter_basic.js')
FORMATTER_QUANTILE = load_script('formatter_quantile.js')


JS_JSON_PARSE = load_script('json_parse.js')

JS_FINANCIAL_TIME_SERIES_0 = load_script('financial_time_series_0.js')
JS_FINANCIAL_TIME_SERIES_TABLE_1 = load_script('financial_time_series_table_1.js')
JS_FINANCIAL_TIME_SERIES_TABLE_2 = load_script('financial_time_series_table_2.js')
HTML_FINANCIAL_TIME_SERIES_TABLE = load_script('financial_time_series_table.html', js=False)
JS_FINANCIAL_TIME_SERIES_TABLE_OPTIONS_1 = load_script('financial_time_series_table_options_1.js')
JS_FINANCIAL_TIME_SERIES_TABLE_OPTIONS_2 = load_script('financial_time_series_table_options_2.js')
JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_0 = 'function(chart) { create_table_0("__uuid__", chart); }'
JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_1 = 'function(chart) { console.log("callback_1 beg "+"__uuid__"); create_table_1("__uuid__", chart); }'
JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_2 = 'function(chart) { create_table_2("__uuid__", chart); }'

TEMPLATE_DISCLAIMER = load_script('template_disclaimer.html', js=False)

PATH_TO_LOGO_JUPYTER = image_src(get_path('Jupyter_logo.png'))
PATH_TO_LOGO_SG = image_src(get_path('SG_logo.png'))




from _wrapper import Wrapper
from _plot import plot, html
from scripts import JS_FINANCIAL_TIME_SERIES_0, \
                    JS_FINANCIAL_TIME_SERIES_TABLE_1, \
                    JS_FINANCIAL_TIME_SERIES_TABLE_2, \
                    JS_FINANCIAL_TIME_SERIES_TABLE_OPTIONS_1, \
                    JS_FINANCIAL_TIME_SERIES_TABLE_OPTIONS_2, \
                    HTML_FINANCIAL_TIME_SERIES_TABLE, \
                    JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_0, \
                    JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_1, \
                    JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_2


class Highstock(Wrapper):
    """
    Main Highstock Object

    API: http://api.highcharts.com/highstock
    Demos: http://www.highcharts.com/stock/demo
    """

    def __init__(self):
        Wrapper.__init__(self, lib='highstock')

        #default config
        self.credits.enabled = False
        self.exporting.enabled = True
        self.chart.animation = False
        
        self.plotOptions.series.animation = False
        self.plotOptions.line.animation = False
        self.plotOptions.column.animation = False


    def html(self, dated=True, save=False, save_name=None, html_init=None,
             js_option_postprocess=None, js_extra=None, callback=None):
        opt = self.to_dict()
        return html(opt, lib='highstock', dated=dated, save=save, save_name=save_name,
            html_init=html_init, js_option_postprocess=js_option_postprocess,
            js_extra=js_extra, callback=callback)


    def plot(self, dated=True, save=False, save_name=None, html_init=None,
             js_option_postprocess=None, js_extra=None, callback=None, footer=None):
        """Only Highstock. No add-on."""
        opt = self.to_dict()
        js_extra = JS_FINANCIAL_TIME_SERIES_0
        callback = JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_0
        
        return plot(opt, lib='highstock', dated=dated, save=save, save_name=save_name,
            html_init=html_init, js_option_postprocess=js_option_postprocess,
            js_extra=js_extra, callback=callback, footer=footer)


    def plot_with_table_1(self, dated=True, save=False, save_name=None, footer=None):
        """
        Table with Perf, IRR, Vol, Sharpe Ratio, Max Drawdown
        Sharpe ratio is based on time series 'Cash' meaning cash compounded
        If no Cash colmun rates are assumed zero.
        """
        opt = self.to_dict()
        html_init = HTML_FINANCIAL_TIME_SERIES_TABLE
        js_option_postprocess = JS_FINANCIAL_TIME_SERIES_TABLE_OPTIONS_1
        js_extra = JS_FINANCIAL_TIME_SERIES_TABLE_1
        callback = JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_1

        return plot(opt, lib='highstock', dated=dated, save=save, save_name=save_name,
            html_init=html_init, js_option_postprocess=js_option_postprocess,
            js_extra=js_extra, callback=callback, footer=footer)


    def plot_with_table_2(self, dated=True, save=False, save_name=None, footer=None):
        """
        Table with Min, Max, Average, Max Drawdown
        """
        opt = self.to_dict()
        html_init = HTML_FINANCIAL_TIME_SERIES_TABLE
        js_option_postprocess = JS_FINANCIAL_TIME_SERIES_TABLE_OPTIONS_2
        js_extra = JS_FINANCIAL_TIME_SERIES_TABLE_2
        callback = JS_FINANCIAL_TIME_SERIES_TABLE_CALLBACK_2

        return plot(opt, lib='highstock', dated=dated, save=save, save_name=save_name,
            html_init=html_init, js_option_postprocess=js_option_postprocess,
            js_extra=js_extra, callback=callback, footer=footer)





import json
from IPython.display import display, HTML, Javascript


JS_SAVE = [
    'https://cdnjs.cloudflare.com/ajax/libs/require.js/2.1.17/require.js',
    'https://cdnjs.cloudflare.com/ajax/libs/jquery/2.1.4/jquery.min.js',
]

JS_LIBS_ONE = json.dumps([
    'jquery',
    'http://code.highcharts.com/stock/highstock.js',
    'http://d3js.org/d3.v3.min.js',
])

JS_LIBS_ONE_sep = ';'.join(map(lambda x: 'console.log("'+x+'")', json.loads(JS_LIBS_ONE)))+';'


JS_LIBS_TWO = json.dumps([
    'https://cdn.datatables.net/1.10.10/js/jquery.dataTables.min.js',
    'http://code.highcharts.com/stock/highcharts-more.js',
    'http://code.highcharts.com/modules/drilldown.js',
    'http://code.highcharts.com/modules/exporting.js',
    'http://code.highcharts.com/modules/heatmap.js',
    'http://code.highcharts.com/modules/treemap.js',
    'http://highcharts.github.io/export-csv/export-csv.js',
])

JS_LIBS_TWO_sep = ';'.join(map(lambda x: 'console.log("'+x+'")', json.loads(JS_LIBS_TWO)))+';'



JS_LOAD = """
require(%s, function() {
    require(%s, function() {
        console.log("The following js libs loaded:");
        console.log("First:");
        %s
        console.log("Then:");
        %s
    });
});""" % (JS_LIBS_ONE, JS_LIBS_TWO, JS_LIBS_ONE_sep, JS_LIBS_TWO_sep)


def load_js_libs():
	display(Javascript(JS_LOAD))



API_DIR = 'api'
SCRIPT_DIR = 'script'
SAMPLES_DIR = 'samples'

HC_OBJECT_FILE = 'hc_object.json'
HC_OPTION_FILE = 'hc_option.json'

HS_OBJECT_FILE = 'hs_object.json'
HS_OPTION_FILE = 'hs_option.json'

FULLNAME = 'fullname'
DESCRIPTION = 'description'
DEMO = 'demo'



DF_ONE_IDX_SEVERAL_COL = 'df_one_idx_several_col.csv'
DF_ONE_IDX_SEVERAL_COL_2 = 'df_one_idx_several_col_2.csv'
DF_ONE_IDX_ONE_COL = 'df_one_idx_one_col.csv'
DF_ONE_IDX_TWO_COL = 'df_one_idx_two_col.csv'
DF_TWO_IDX_ONE_COL = 'df_two_idx_one_col.csv'
DF_SCATTER = 'df_scatter.csv'
DF_BUBBLE = 'df_bubble.csv'
DF_HEATMAP = 'df_heatmap.csv'
DF_SEVERAL_IDX_ONE_COL = 'df_several_idx_one_col.csv'





import os
import numpy as np
import pandas as pd

from _config import SAMPLES_DIR, DF_ONE_IDX_SEVERAL_COL, DF_ONE_IDX_SEVERAL_COL_2, \
                    DF_ONE_IDX_ONE_COL, DF_ONE_IDX_TWO_COL, DF_TWO_IDX_ONE_COL, DF_SCATTER, \
                    DF_BUBBLE, DF_HEATMAP, DF_SEVERAL_IDX_ONE_COL


def load_df(src):
    _dir = os.path.dirname(__file__)
    df_file = os.path.join(_dir, SAMPLES_DIR, src)
    df = pd.read_csv(df_file)
    return df



def df_timeseries(N=3, Nb_bd=500, seed=123456):
    np.random.seed(seed)
    
    rate = 0.02
    vol = 0.25
    dt = 1.0/260
    tracks = np.zeros([Nb_bd, N], dtype=np.float)

    for k in range(N):
        ln_returns = (rate-vol**2/2)*dt+vol*np.sqrt(dt)*np.random.normal(size=Nb_bd)
        ln_returns[0] = 0.0
        tracks[:, k] = np.exp(ln_returns).cumprod()
        
    dates = pd.date_range(start=pd.datetime(2015, 1, 1), periods=Nb_bd, freq='B')
    df = pd.DataFrame(data=tracks, index=dates, columns=['Track'+str(1+i) for i in range(N)])
    
    return df



def df_one_idx_several_col():
    df = load_df(DF_ONE_IDX_SEVERAL_COL)
    df = df.set_index('Fruit')
    return df

def df_one_idx_several_col_2():
    df = load_df(DF_ONE_IDX_SEVERAL_COL_2)
    df = df.set_index('WeekDay')
    return df


def df_one_idx_one_col():
    df = load_df(DF_ONE_IDX_ONE_COL)
    df = df.set_index('Brand')
    return df


def df_one_idx_two_col():
    df = load_df(DF_ONE_IDX_TWO_COL)
    df = df.set_index('Month')
    return df


def df_two_idx_one_col():
    df = load_df(DF_TWO_IDX_ONE_COL)
    df = df.set_index(['Brand', 'Version'])
    return df


def df_scatter():
    df = load_df(DF_SCATTER)
    df = df.set_index(['Height', 'Weight'])
    return df


def df_bubble():
    df = load_df(DF_BUBBLE)
    df = df.set_index(['Cat', 'x', 'y'])
    return df


def df_heatmap():
    df = load_df(DF_HEATMAP)
    df = df.set_index(['Name', 'Day'])
    return df


def df_several_idx_one_col():
    df = load_df(DF_SEVERAL_IDX_ONE_COL)
    df = df.set_index(['Region', 'Country', 'Cause'])
    df = df.sortlevel()
    return df







from _highcharts import Highcharts



class Clock(Highcharts):
    """
    Highcharts Clock
    http://jsfiddle.net/gh/get/jquery/1.9.1/highslide-software/highcharts.com/tree/master/samples/highcharts/demo/gauge-clock/
    """

    def __init__(self):

        self.js_extra = """
        function getNow() {
            var now = new Date();

            return {
                hours: now.getHours() + now.getMinutes() / 60,
                minutes: now.getMinutes() * 12 / 60 + now.getSeconds() * 12 / 3600,
                seconds: now.getSeconds() * 12 / 60
            };
        }

        var now = getNow();

        function pad(number, length) {
            // Create an array of the remaining length + 1 and join it with 0's
            return new Array((length || 2) + 1 - String(number).length).join(0) + number;
        }


        // Extend jQuery with some easing (copied from jQuery UI)
        $.extend($.easing, {
            easeOutElastic: function (x, t, b, c, d) {
                var s=1.70158;var p=0;var a=c;
                if (t==0) return b;  if ((t/=d)==1) return b+c;  if (!p) p=d*.3;
                if (a < Math.abs(c)) { a=c; var s=p/4; }
                else var s = p/(2*Math.PI) * Math.asin (c/a);
                return a*Math.pow(2,-10*t) * Math.sin( (t*d-s)*(2*Math.PI)/p ) + c + b;
            }
        });
        """

        self.callback = """
        function (chart) {
            setInterval(function () {

                now = getNow();

                var hour = chart.get('hour'),
                    minute = chart.get('minute'),
                    second = chart.get('second'),
                    // run animation unless we're wrapping around from 59 to 0
                    animation = now.seconds === 0 ? false : { easing: 'easeOutElastic' };

                // Cache the tooltip text
                chart.tooltipText =
                    pad(Math.floor(now.hours), 2) + ':' +
                    pad(Math.floor(now.minutes * 5), 2) + ':' +
                    pad(now.seconds * 5, 2);

                hour.update(now.hours, true, animation);
                minute.update(now.minutes, true, animation);
                second.update(now.seconds, true, animation);

            }, 1000);
        }
        """


        self.g = Highcharts()

        self.g.chart = {
            'type': 'gauge',
            'plotBackgroundColor': None,
            'plotBackgroundImage': None,
            'plotBorderWidth': 0,
            'plotShadow': False,
            'height': 250
        }

        self.g.title.text = 'The Highcharts Clock'

        self.g.pane.background = [
            # default background
            {}, 
            # reflex for supported browsers
            {'backgroundColor': {
                'radialGradient': {
                    'cx': 0.5,
                    'cy': -0.4,
                    'r': 1.9
                },
                'stops': [
                    [0.5, 'rgba(255, 255, 255, 0.2)'],
                    [0.5, 'rgba(200, 200, 200, 0.2)']
                ]
            }}
        ]

        self.g.yAxis = {
            'labels': { 'distance': -20 },
            'min': 0,
            'max': 12,
            'lineWidth': 0,
            'showFirstLabel': False,

            'minorTickInterval': 'auto',
            'minorTickWidth': 1,
            'minorTickLength': 5,
            'minorTickPosition': 'inside',
            'minorGridLineWidth': 0,
            'minorTickColor': '#666',

            'tickInterval': 1,
            'tickWidth': 2,
            'tickPosition': 'inside',
            'tickLength': 10,
            'tickColor': '#666',
        }

        self.g.tooltip.formatter = 'function() { return this.series.chart.tooltipText; }'

        self.g.series = [{
            'data': [{
                'id': 'hour',
                'y': '(function() { var now = new Date(); return now.getHours() + now.getMinutes() / 60; })()',
                'dial': {
                    'radius': '60%',
                    'baseWidth': 4,
                    'baseLength': '95%',
                    'rearLength': 0
                }
            }, {
                'id': 'minute',
                'y': '(function() { var now = new Date(); return now.getMinutes() * 12 / 60 + now.getSeconds() * 12 / 3600; })()',
                'dial': {
                    'baseLength': '95%',
                    'rearLength': 0
                }
            }, {
                'id': 'second',
                'y': '(function() { var now = new Date(); return now.getSeconds() * 12 / 60; })()',
                'dial': {
                    'radius': '100%',
                    'baseWidth': 1,
                    'rearLength': '20%'
                }
            }],
            'animation': False,
            'dataLabels': {
                'enabled': False
            }
        }]





    def _repr_html_(self):
        return self.g.html(js_extra=self.js_extra, callback=self.callback)



